// Fill out your copyright notice in the Description page of Project Settings.

#include "MyBlueprintFunctionLibrary.h"
#include "MyProject/MySettings/MySettings.h"
#include "Misc/Paths.h"

#if WITH_EDITOR
#include "Engine/DataTable.h"
#include "Serialization/JsonWriter.h"
#include "Misc/FileHelper.h"
#include "JsonObjectConverter.h"
#include "SourceControlHelpers.h"
#endif

FString UMyBlueprintFunctionLibrary::GetExportFileName(const UObject *Object, const FString &Extension)
{
	// ẘgqŁAUObject ƓÕt@C𐶐
	// t@CpX́AvWFNgݒɈˑ
	static const FString PathPrefix = TEXT("/Game");
	const UMySettings *Settings = UMySettings::GetMySettings();
	if (Settings != nullptr) {
		const FString ProjectDir = FPaths::ProjectDir();
		FString FileName = Object->GetPackage()->GetFName().ToString() + TEXT(".") + Extension;
		if (FileName.StartsWith(PathPrefix))
		{
			FileName = FileName.RightChop(PathPrefix.Len());
		}
		const FString OutputPath = FPaths::Combine(ProjectDir, Settings->ResourceOutputFolder, FileName);
		return OutputPath;
	}

	return FString();
}

void UMyBlueprintFunctionLibrary::ImportJsonToObject(const FString &FilePath, UObject *OutObject)
{
	// JSON t@C UObject 𐶐
#if WITH_EDITOR
	if (!OutObject)
	{
		return;
	}

	FString JsonString;
	if (LoadFileToString(FilePath, JsonString))
	{
		TSharedRef<TJsonReader<TCHAR>> JsonReader = TJsonReaderFactory<TCHAR>::Create(JsonString);
		TSharedPtr<FJsonObject> JsonObject = MakeShareable(new FJsonObject());
		FJsonSerializer::Deserialize(JsonReader, JsonObject);
		if (!FJsonObjectConverter::JsonObjectToUStruct(JsonObject.ToSharedRef(), OutObject->GetClass(), (void*)(OutObject), 0, 0))
		{
			UE_LOG(LogTemp, Error, TEXT("UMyBlueprintFunctionLibrary::ImportJsonToObject: Failed to Import %s"), *OutObject->GetName());
		}
	}
	else
	{
		UE_LOG(LogTemp, Error, TEXT("UMyBlueprintFunctionLibrary::ImportJsonToObject: Failed to Load %s"), *FilePath);
	}
#endif
}

void UMyBlueprintFunctionLibrary::ExportObjectAsJson(const UObject *Object, const FString &OutputPath)
{
	// JSON ` UObject o
#if WITH_EDITOR
	if (!Object)
	{
		return;
	}

	FString JsonString;
	if (UObjectToJsonString(Object, Object->GetClass(), JsonString))
	{
		SaveStringToFile(JsonString, OutputPath);
	}
	else
	{
		UE_LOG(LogTemp, Error, TEXT("UMyBlueprintFunctionLibrary::ExportObjectAsJson: Failed to Export %s"), *Object->GetName());
	}
#endif
}

void UMyBlueprintFunctionLibrary::ExportDataTableAsCSV(const UDataTable *Table)
{
	// CSV `Ńf[^e[uo
#if WITH_EDITOR
	if (!Table)
	{
		return;
	}

	const FString CsvString = Table->GetTableAsCSV(EDataTableExportFlags::None);
	const FString OutputPath = GetExportFileName(Table, TEXT("csv"));
	SaveStringToFile(CsvString, OutputPath);
#endif
}

void UMyBlueprintFunctionLibrary::SaveStringToFile(const FString &String, const FString &FilePath)
{
	// UTF-8 ݒŃeLXgt@Co
#if WITH_EDITOR
	if (USourceControlHelpers::IsEnabled())
	{
		USourceControlHelpers::CheckOutFile(FilePath);
	}
#endif

	FFileHelper::SaveStringToFile(String, *FilePath, FFileHelper::EEncodingOptions::ForceUTF8);

#if WITH_EDITOR
	if (USourceControlHelpers::IsEnabled())
	{
		USourceControlHelpers::MarkFileForAdd(FilePath);
	}
#endif
}

bool UMyBlueprintFunctionLibrary::LoadFileToString(const FString &FilePath, FString &OutString)
{
	return FFileHelper::LoadFileToString(OutString, *FilePath, FFileHelper::EHashOptions::None, 0);
}

bool UMyBlueprintFunctionLibrary::UObjectToJsonString(const UObject *Object, UStruct *ObjectStruct, FString &OutJsonString)
{
#if WITH_EDITOR
	//  UObject  Json ɕϊ
	TSharedPtr<FJsonObject> JsonObject = MakeShareable(new FJsonObject());
	FJsonObjectConverter::UStructToJsonObject(ObjectStruct, (void*)(Object), JsonObject.ToSharedRef(), 0, 0, nullptr);
	TSharedRef<TJsonWriter<TCHAR, TPrettyJsonPrintPolicy<TCHAR>>> JsonWriter = TJsonWriterFactory<TCHAR, TPrettyJsonPrintPolicy<TCHAR>>::Create(&OutJsonString, 0);
	return FJsonSerializer::Serialize(JsonObject.ToSharedRef(), JsonWriter);
#endif
	return false;
}

bool UMyBlueprintFunctionLibrary::JsonStringToUObject(FString &JsonString, UStruct *ObjectStruct, const UObject *OutObject)
{
#if WITH_EDITOR
	// Json 񂩂 UObject ɕϊ
	if (!OutObject)
	{
		return false;
	}
	TSharedRef<TJsonReader<TCHAR>> JsonReader = TJsonReaderFactory<TCHAR>::Create(JsonString);
	TSharedPtr<FJsonObject> JsonObject = MakeShareable(new FJsonObject());
	FJsonSerializer::Deserialize(JsonReader, JsonObject);

	if (FJsonObjectConverter::JsonObjectToUStruct(JsonObject.ToSharedRef(), ObjectStruct, (void*)OutObject, 0, 0))
	{
		OutObject->GetPackage()->MarkPackageDirty();
		return true;
	}
#endif
	return false;
}
