// Fill out your copyright notice in the Description page of Project Settings.

#if WITH_EDITOR
#include "AssetTypeActions_MyDataAsset.h"
#include "MyDataAsset.h"
#include "EditorFramework/AssetImportData.h"

#define LOCTEXT_NAMESPACE "AssetTypeActions"

UClass* FAssetTypeActions_MyDataAsset::GetSupportedClass() const
{
	return UMyDataAsset::StaticClass();
}

void FAssetTypeActions_MyDataAsset::GetResolvedSourceFilePaths(const TArray<UObject*>& TypeAssets, TArray<FString>& OutSourceFilePaths) const
{
	for (UObject *TypeAsset : TypeAssets)
	{
		const UMyDataAsset *MyDataAsset = CastChecked<UMyDataAsset>(TypeAsset);
		if(MyDataAsset != nullptr && MyDataAsset->AssetImportData != nullptr)
		{
			MyDataAsset->AssetImportData->ExtractFilenames(OutSourceFilePaths);
		}
	}
}

void FAssetTypeActions_MyDataAsset::GetActions(const TArray<UObject*>& InObjects, FMenuBuilder& MenuBuilder)
{
	TArray<TWeakObjectPtr<UMyDataAsset>> MyAssetImports = GetTypedWeakObjectPtrs<UMyDataAsset>(InObjects);
	MenuBuilder.AddMenuEntry(
		LOCTEXT("MyDataAsset_Export", "Export"),
		LOCTEXT("MyDataAsset_ExportTooltip", "Export this MyDataAsset"),
		FSlateIcon(),
		FUIAction(FExecuteAction::CreateSP(this, &FAssetTypeActions_MyDataAsset::Export, MyAssetImports), FCanExecuteAction())
	);
}

void FAssetTypeActions_MyDataAsset::Export(TArray<TWeakObjectPtr<UMyDataAsset>> Objects)
{
	for (auto It = Objects.CreateIterator(); It; ++It)
	{
		UMyDataAsset *MyDataAsset = (*It).Get();
		if (MyDataAsset != nullptr)
		{
			MyDataAsset->Export();
		}
	}
}
#undef LOCTEXT_NAMESPACE
#endif
