// Fill out your copyright notice in the Description page of Project Settings.

#if WITH_EDITOR
#include "MyDataAssetFactory.h"
#include "MyDataAsset.h"
#include "EditorFramework/AssetImportData.h"
#include "EditorReimportHandler.h"
#include "JsonObjectConverter.h"

UMyDataAssetFactory::UMyDataAssetFactory(const FObjectInitializer& ObjectInitializer)
	: Super(ObjectInitializer)
{
	SupportedClass = UMyDataAsset::StaticClass();
	bCreateNew = false;
	bEditorImport = true;
	bText = true;
	Formats.Add(UMyDataAsset::FileExtension + TEXT(";My Data Asset"));
}

bool UMyDataAssetFactory::DoesSupportClass(UClass* Class)
{
	return Class->IsChildOf(UMyDataAsset::StaticClass());
}

UClass* UMyDataAssetFactory::ResolveSupportedClass()
{
	return UMyDataAsset::StaticClass();
}

UObject* UMyDataAssetFactory::FactoryCreateText(UClass* InClass, UObject* InParent, FName InName, EObjectFlags Flags, UObject* Context, const TCHAR* Type, const TCHAR*& Buffer, const TCHAR* BuferEnd, FFeedbackContext* Warn)
{
	UClass *Class = InClass;
	TSharedRef<TJsonReader<TCHAR>> JsonReader = TJsonReaderFactory<TCHAR>::Create(Buffer);
	TSharedPtr<FJsonObject> JsonObject = MakeShareable(new FJsonObject());
	FJsonSerializer::Deserialize(JsonReader, JsonObject);
	TSharedPtr<FJsonValue> *ClassName = (*JsonObject).Values.Find(TEXT("nativeClass"));
	if (ClassName != nullptr)
	{
		UClass *FoundClass = FindObject<UClass>(ANY_PACKAGE, *(*ClassName).Get()->AsString(), false);
		if (FoundClass != nullptr)
		{
			Class = FoundClass;
		}
		FStaticConstructObjectParameters Params(Class);
		Params.Outer = InParent;
		Params.Name = InName;
		Params.SetFlags = Flags;
		UMyDataAsset* NewMyAsset = CastChecked<UMyDataAsset>(StaticConstructObject_Internal(Params));
		FJsonObjectConverter::JsonObjectToUStruct(JsonObject.ToSharedRef(), Class, (void*)(NewMyAsset), 0, 0);

		return NewMyAsset;
	}

	UE_LOG(LogTemp, Error, TEXT("UMyDataAssetFactory::FactoryCreateText: nativeClass was not found"));
	return nullptr;
}

bool UMyDataAssetFactory::CanReimport(UObject* Obj, TArray<FString>& OutFilenames)
{
	UMyDataAsset* MyAsset = Cast<UMyDataAsset>(Obj);
	if (MyAsset != nullptr && MyAsset->AssetImportData != nullptr)
	{
		MyAsset->AssetImportData->ExtractFilenames(OutFilenames);
		return true;
	}
	return false;
}

void UMyDataAssetFactory::SetReimportPaths(UObject* Obj, const TArray<FString>& NewReimportPaths)
{
	UMyDataAsset* MyAsset = Cast<UMyDataAsset>(Obj);
	if (MyAsset && ensure(NewReimportPaths.Num() == 1))
	{
		MyAsset->AssetImportData->UpdateFilenameOnly(NewReimportPaths[0]);
	}
}

EReimportResult::Type UMyDataAssetFactory::Reimport(UObject* Obj)
{
	UMyDataAsset* MyAsset = Cast<UMyDataAsset>(Obj);
	if (!MyAsset)
	{
		return EReimportResult::Failed;
	}

	const FString Filename = MyAsset->AssetImportData->GetFirstFilename();
	if (!Filename.Len() || IFileManager::Get().FileSize(*Filename) == INDEX_NONE)
	{
		return EReimportResult::Failed;
	}

	EReimportResult::Type Result = EReimportResult::Failed;
	if (UFactory::StaticImportObject(MyAsset->GetClass(), MyAsset->GetOuter(), *MyAsset->GetName(), RF_Public | RF_Standalone, *Filename, NULL, this))
	{
		if (MyAsset->GetOuter())
		{
			MyAsset->GetOuter()->MarkPackageDirty();
		}
		else
		{
			MyAsset->MarkPackageDirty();
		}
		return EReimportResult::Succeeded;
	}

	return EReimportResult::Failed;
}
#endif
